#include "gamec.h"

void do_cycle(GameControl** gamec) {

	GList* walker;
	
	if(CHK((*gamec)->game_state,GAME_FREEZE) == 0) {

		update_area_spaces(gamec);
		repaint_drawing_area(gamec);

		walker = (*gamec)->monsters;

		while(walker != NULL) {

			if(walker->data != NULL) {
			
				(*gamec)->turn = ((Monster*)walker->data);

				if(CHK((*gamec)->turn->properties,IS_PLAYER)) {
			
					parse_key_stack(gamec);
			
				} else {
		
					(*gamec)->turn->behaviour(gamec,(*gamec)->turn);
			
				}

			}

			walker = walker->next;

		}

		/* causes overflow after a while, 
		   so take care when dealing with a future cycle,
		   use the INCR_CYCLE(N) macro to get the cycle number N steps ahead.
		*/
		(*gamec)->cycle_cnt++;
	
	} else { /* GAME_FREEZE == 1 */

	}

}

GameControl* new_gamec() {
	
	GameControl* new = (GameControl*) g_malloc(sizeof(GameControl));

	new->map = (Map*)new_map();
	new->map_window = (MapWin*)new_map_window();
	new->image_files = NULL;
	new->sprites = NULL;
	new->key_stack = NULL;
	new->items = NULL;
	new->local_items = NULL;
	new->monsters = NULL;
	new->local_monsters = NULL;
	new->turn = 0;
	new->game_state = 0;
	new->cycle_cnt = 1;

	return new;

}

void zap_gamec(GameControl* gamec) {

	if(gamec != NULL) {
		
		if(gamec->map != NULL) {
			
			zap_map(gamec->map);
		
		}

		if(gamec->map_window != NULL) {
			
			zap_map_window(gamec->map_window);
		
		}

		if(gamec->sprites != NULL) {
			
			g_list_foreach(gamec->sprites,(GFunc)zap_sprite,NULL);
			g_list_free(gamec->sprites);
			gamec->sprites = NULL;
		
		}
		
		if(gamec->image_files != NULL) {

			/* I don't free these strings because they're not dynamically allocated */
			g_list_free(gamec->image_files);
			gamec->image_files = NULL;

		}

		if(gamec->monsters != NULL) {

			g_list_foreach(gamec->monsters,(GFunc)zap_monster,NULL);
			g_list_free(gamec->monsters);
			gamec->monsters = NULL;
		
		}

		if(gamec->local_monsters != NULL) {

			g_list_free(gamec->local_monsters);
			gamec->local_monsters = NULL;
		
		}

		if(gamec->items != NULL) {

			g_list_foreach(gamec->items,(GFunc)zap_item,NULL);
			g_list_free(gamec->items);
			gamec->items = NULL;
		
		}

		if(gamec->local_items != NULL) {

			g_list_free(gamec->local_items);
			gamec->local_items = NULL;
		
		}
		
		g_free(gamec);
		gamec = NULL;

	}

}

void key_interface(GtkWidget* widget,GdkEventKey* event,GameControl** gamec) {

	gint n;

	if(event->type == GDK_KEY_PRESS) {

		switch(event->keyval) {

			case GDK_KP_Up:
			case GDK_KP_Page_Up:
			case GDK_KP_Right:
			case GDK_KP_Page_Down:
			case GDK_KP_Down:
			case GDK_KP_End:
			case GDK_KP_Left:
			case GDK_KP_Home:
				if((n = g_list_length((*gamec)->key_stack)) == 0) {
	
					(*gamec)->key_stack = g_list_append((*gamec)->key_stack,GINT_TO_POINTER(event->keyval));
	
				}
				break;
			/* this is a list of keyvals that's empty now,
			   because it's only used for commands... 
		           (e.g. M+Up+Right = move thing above you to the right)
		           and there are no commands yet.
			*/
			default:
				break;

		}

	}

}

void mouse_interface(GtkWidget* widget,GdkEventButton* event,GameControl** gamec) {

	g_print("mouse interface event %d\n",event->type);

}

void freeze_game(GameControl** gamec) {

	(*gamec)->game_state = (CHK((*gamec)->game_state,GAME_FREEZE)) ?
	                       (CLR((*gamec)->game_state,GAME_FREEZE)) :
	                       (SET((*gamec)->game_state,GAME_FREEZE));

}