#include "map.h"

Map* new_map() {

	Map* new = (Map*)g_malloc(sizeof(Map));
	gint i,j;

	new->name = g_string_new("new map");

	for(j=0;j<MAX_MAP_Y;j++)
	for(i=0;i<MAX_MAP_X;i++)
		new->cell[i][j] = new_space();

	return new;

}

void zap_map(Map* map) {

	gint x,y;

	if(map != NULL) {

		if(map->name != NULL) {
		
			g_string_free(map->name,TRUE);
			map->name = NULL;
		
		}		

		for(y=0;y<MAX_MAP_Y;y++)
		for(x=0;x<MAX_MAP_X;x++)
			if(map->cell[x][y] != NULL) {

				zap_space(map->cell[x][y]);

			}

		g_free(map);
		map = NULL;

	}

}

MapWin* new_map_window() {
	
	MapWin* new = (MapWin*)g_malloc(sizeof(MapWin));

	new->window = NULL;
	new->box = NULL;
	new->menu = NULL;
	new->drawing_area = NULL;
	new->area = NULL;
	new->gc = NULL;
	new->update = NULL;
	new->area_ox = 0;
	new->area_oy = 0;

	return new;

}

void zap_map_window(MapWin* map_window) {

	if(map_window->update != NULL) {

		g_list_foreach(map_window->update,(GFunc)zap_coords,NULL);
		g_list_free(map_window->update);
		
	}

	gdk_pixmap_unref(map_window->area);
	gdk_gc_unref(map_window->gc);

	if(map_window != NULL) {

		g_free(map_window);
		map_window = NULL;

	}

}

Space* new_space() {
	
	Space* new = (Space*)g_malloc(sizeof(Space));

	new->kind = UNKNOWN_KIND;
	new->variation = 0;
	new->sprites = NULL;
	new->animates = FALSE;	
	new->sprite_cur = 0;
	new->mos = NULL;
	new->ios = NULL;

	return new;

}

void zap_space(Space* space) {
	
	/* items, monsters and sprites get g_free'ed with the GameControl */
	
	if(space != NULL) {
		
		if(space->sprites != NULL) {

			g_list_foreach(space->sprites,(GFunc)g_free,NULL);
			space->sprites = NULL;

		}

		g_free(space);
		space = NULL;

	}

}

void set_space(GameControl** gamec,Space* space,gint16 kind,gint16 variation) {

	space->kind = kind;
	space->variation = variation;
	
	switch(kind) {

		case GRASS:
			space->animates = FALSE;
			if(variation == 0) {	
				space->sprites = add_sprite(space->sprites,FIRST_FRAME,GRASS_1);
				space->sprite_cur = FIRST_FRAME; 
			} else if(variation == 1) {
				space->sprites = add_sprite(space->sprites,FIRST_FRAME,GRASS_2);
				space->sprite_cur = FIRST_FRAME;
			} else {
				space->sprites = add_sprite(space->sprites,FIRST_FRAME,UNKNOWN_SPACE_SPRITE);
				space->sprite_cur = FIRST_FRAME;
			} break;

		case DIRT:
			space->animates = FALSE;
			if(variation == 0) {
				space->sprites = add_sprite(space->sprites,FIRST_FRAME,DIRT_1);
				space->sprite_cur = FIRST_FRAME;
			} else if(variation == 1) {
				space->sprites = add_sprite(space->sprites,FIRST_FRAME,DIRT_2);
				space->sprite_cur = FIRST_FRAME;
			} else {
				space->sprites = add_sprite(space->sprites,FIRST_FRAME,UNKNOWN_SPACE_SPRITE);
				space->sprite_cur = FIRST_FRAME;
			} break;

		default:
			space->animates = FALSE;
			space->sprites = add_sprite(space->sprites,FIRST_FRAME,UNKNOWN_SPACE_SPRITE);
			space->sprite_cur = FIRST_FRAME;

	}

}

Coords* new_coords(gint x, gint y) {
	
	Coords* new = (Coords*)g_malloc(sizeof(Coords));

	new->type = COORDS;
	new->x = x;
	new->y = y;

	return new;

}

void zap_coords(Coords* coords) {

	if(coords != NULL) {

		g_free(coords);
		coords = NULL;

	}

}

gint update_compare(gconstpointer a,gconstpointer b) {

	if(a != NULL) {
		
		if((((Coords*)a)->x == ((Coords*)b)->x) && 
		   (((Coords*)a)->y == ((Coords*)b)->y)) {
	
			return 0;
	
		}
		
	}

	return 1;

}

void add_update(GameControl** gamec,gint x,gint y) {
	
	Coords* new = new_coords(x,y);

	if(g_list_find_custom(MAP_UPDATE,new,update_compare) == NULL) {
	
		MAP_UPDATE = g_list_append(MAP_UPDATE,new);

	}

}

void add_to_map(GameControl** gamec,Monster* monster,gint x,gint y) {

	gint i,j;

	monster->x = x;
	monster->y = y;

	for(j=0;j<monster->height;j++)
	for(i=0;i<monster->width;i++) {
		
		if(monster->type == MONSTER) {
	
			MAP(x+i,y+j)->mos = monster;

		} else if(monster->type == ITEM) {

			MAP(x+i,y+j)->ios = (Item*)monster;

		}

	}

}

void remove_from_map(GameControl** gamec,Monster* monster) {

	gint i,j;		

	for(j=0;j<monster->height;j++)
	for(i=0;i<monster->width;i++) {
		if(monster->type == MONSTER) {

			MAP(monster->x+i,monster->y+j)->mos = NULL;
	
		} else if(monster->type == ITEM) {

			MAP(monster->x+i,monster->y+j)->ios = MAP(monster->x+i,monster->y+j)->ios->stacked;

		}

	}

}


void random_fill_map(GameControl** gamec) {

	gint x,y,rn;
	Item* item;
	Monster* monster;

	srand(time(NULL));

	for(y=0;y<MAX_MAP_Y;y++)
	for(x=0;x<MAX_MAP_X;x++) {

		MAP(x,y)->mos = NULL;
		MAP(x,y)->ios = NULL;

		set_space(gamec,MAP(x,y),GRASS,rand()%2);
		
		if((rn=(rand()%60))<9) {

			item = new_item();

			if(rn == 8) {

				set_item(gamec,item,BIG_BOULDER,"a big boulder");

			} else if(rn == 7) {

				set_item(gamec,item,MOSSY_BOULDER,"some mossy boulders");

			} else if(rn == 6) {

				set_item(gamec,item,BIG_TREE,"a tree");

			} else if(rn == 5) {

				set_item(gamec,item,SMALL_TREE,"a little tree");

			} else if(rn == 4) {

				set_item(gamec,item,DARK_BARREL,"a barrel");

			} else if(rn == 3) {

				set_item(gamec,item,LIGHT_BARREL,"a new barrel");

			} else if(rn == 2) {

				set_item(gamec,item,RED_RUG,"an old red rug");

			} else if(rn == 1) {

				set_item(gamec,item,PURPLE_RUG,"a nice purple rug");

			} else if(rn == 0){

				set_item(gamec,item,CAMP_FIRE,"a cozy campfire");

			}

			add_to_map(gamec,(Monster*)item,x,y);
			
		}
		
	}
	
	MAP(2,2)->ios = NULL;
	MAP(2,3)->ios = NULL;
	MAP(3,2)->ios = NULL;
	MAP(3,3)->ios = NULL;

	MAP(2,4)->ios = NULL;
	MAP(3,4)->ios = NULL;

	monster = new_monster();
	set_monster((gamec),monster,BRUTE_OGRE,"Krunk the ogre");
	monster->behaviour = behave_guard_up_down;
	monster->memory = set_memory(monster->memory,1,INT_MEM,DOWN,NULL);
	add_to_map((gamec),monster,2,2);

	MAP(7,5)->ios = NULL;

	MAP(7,4)->ios = NULL;

	monster = new_monster();
	set_monster((gamec),monster,KNIGHT_HUMAN,"Sir Strandarr");
	monster->behaviour = behave_guard_up_down;
	monster->memory = set_memory(monster->memory,1,INT_MEM,DOWN,NULL);
	monster->properties = SET(monster->properties,IS_PLAYER);
	add_to_map((gamec),monster,7,5);

	MAP(10,10)->ios = NULL;
	MAP(10,11)->ios = NULL;
	MAP(11,10)->ios = NULL;
	MAP(11,11)->ios = NULL;

	item = new_item();
	set_item((gamec),item,MUSHROOM_RING_FUNGUS,"a circle of mushrooms");
	add_to_map((gamec),(Monster*)item,10,10);

}
