#include "xfuncs.h"

void init_widgets(GameControl** gamec) {

	MAP_WINDOW = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(MAP_WINDOW),QUEST_WINDOW_TITLE);
	gtk_container_set_border_width(GTK_CONTAINER(MAP_WINDOW),0);

	gtk_widget_set_events(MAP_WINDOW,GDK_KEY_PRESS_MASK    | GDK_KEY_RELEASE_MASK | 
	                                 GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK );
	gtk_signal_connect(GTK_OBJECT(MAP_WINDOW),"delete_event",GTK_SIGNAL_FUNC(delete_event),NULL); 
	gtk_signal_connect(GTK_OBJECT(MAP_WINDOW),"destroy",GTK_SIGNAL_FUNC(destroy),NULL);
	gtk_signal_connect(GTK_OBJECT(MAP_WINDOW),"key_press_event",GTK_SIGNAL_FUNC(key_interface),gamec);
	gtk_signal_connect(GTK_OBJECT(MAP_WINDOW),"key_release_event",GTK_SIGNAL_FUNC(key_interface),gamec);
	gtk_signal_connect(GTK_OBJECT(MAP_WINDOW),"button_press_event",GTK_SIGNAL_FUNC(mouse_interface),gamec);
	gtk_signal_connect(GTK_OBJECT(MAP_WINDOW),"button_release_event",GTK_SIGNAL_FUNC(mouse_interface),gamec);
	
	MAP_BOX = gtk_vbox_new(FALSE,0);

	get_map_menu(gamec);
	gtk_box_pack_start(GTK_BOX(MAP_BOX),MAP_MENU,FALSE,TRUE,0);
	gtk_widget_show(MAP_MENU);

	MAP_DRAWING_AREA = gtk_drawing_area_new();
	gtk_drawing_area_size(GTK_DRAWING_AREA(MAP_DRAWING_AREA), 360, 360);
	gtk_box_pack_start(GTK_BOX(MAP_BOX),MAP_DRAWING_AREA,TRUE,TRUE,0);
	gtk_signal_connect(GTK_OBJECT(MAP_DRAWING_AREA),"configure_event",GTK_SIGNAL_FUNC(configure_event),gamec);
	gtk_signal_connect_after(GTK_OBJECT(MAP_DRAWING_AREA),"expose_event",GTK_SIGNAL_FUNC(expose_event),gamec);
	gtk_widget_show(MAP_DRAWING_AREA);
	
	gtk_container_add(GTK_CONTAINER(MAP_WINDOW),MAP_BOX);
	gtk_widget_show(MAP_BOX);
	
	gtk_widget_show(MAP_WINDOW);

	g_timeout_add(GAME_CYCLE_TIME,(GtkFunction)do_cycle,gamec);

}

void print_image_list(GameControl** gamec) {
	
	GList* walker = (*gamec)->image_files;
	gint i=0;

	g_print("image list:\n");

	while(walker != NULL) {

		if(walker->data != NULL) {
			
			g_print("%d: %s\n",i,((gchar*)walker->data));

		} else {

			g_print("aargh! there's a NULL node in my image list!\n");

		}

		i++;
		walker = walker->next;

	}

}

void print_monster_list(GameControl** gamec) {
	
	GList* walker = (*gamec)->monsters;
	gint i=0;

	g_print("monster list:\n");

	while(walker != NULL) {

		if(walker->data != NULL) {
			
			g_print("%d: %s\n",i,((Monster*)walker->data)->name->str);

		} else {

			g_print("aargh! there's a NULL node in my monster list!\n");

		}

		i++;
		walker = walker->next;

	}

}

void print_item_list(GameControl** gamec) {
	
	GList* walker = (*gamec)->items;
	gint i=0;

	g_print("item list:\n");

	while(walker != NULL) {

		if(walker->data != NULL) {
			
			g_print("%d: %s\n",i,((Item*)walker->data)->name->str);

		} else {

			g_print("aargh! there's a NULL node in my item list!\n");

		}

		i++;
		walker = walker->next;

	}

}

void print_key_stack(GameControl** gamec) {
	
	GList* walker = (*gamec)->key_stack;
	gint i=0;

	g_print("key stack:\n");

	while(walker != NULL) {

		if(walker->data != NULL) {
			
			g_print("%d: %d (%s)\n",i,(gint)walker->data,gdk_keyval_name((gint)walker->data));

		} else {

			g_print("aargh! there's a NULL node in my item stack!\n");

		}

		i++;
		walker = walker->next;

	}

}

void print_cycle_cnt(GameControl** gamec) {

	g_print("%d\n",CYCLE_CNT);
}

void get_map_menu(GameControl** gamec) {

	GtkItemFactory *item_factory;
	GtkAccelGroup *accel_group;
	GtkItemFactoryEntry menu_items[] = {
		{"/_Game",NULL,NULL,0,"<Branch>"},
		{"/Game/Quit","<control>Q",gtk_main_quit,0,NULL},
		{"/_Debug",NULL,NULL,0,"<Branch>"},
		{"/Debug/Print Cycle",NULL,print_cycle_cnt,0,NULL},
		{"/Debug/Print Image List",NULL,print_image_list,0,NULL},
		{"/Debug/Print Monster List",NULL,print_monster_list,0,NULL},
		{"/Debug/Print Item List",NULL,print_item_list,0,NULL},
		{"/Debug/Print Key Stack",NULL,print_key_stack,0,NULL},
		{"/_Commands",NULL,NULL,0,"<Branch>"},
		{"/Commands/(un)Freeze Game","f",freeze_game,0,NULL},
		{"/_Help",NULL,NULL,0,"<LastBranch>"},
		{"/_Help/About",NULL,about,0,NULL}
	};
	gint nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]);

	accel_group = gtk_accel_group_new ();
	item_factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR,"<main>",accel_group);

	gtk_item_factory_create_items(item_factory,nmenu_items,menu_items,gamec);
	gtk_accel_group_attach(accel_group, GTK_OBJECT(MAP_WINDOW));
	
	MAP_MENU = gtk_item_factory_get_widget(item_factory,"<main>");

}

gint delete_event(GtkWidget *widget,GdkEvent *event,gpointer data) {

	/* turn TRUE into FALSE and the window will get destroyed */
	return FALSE;

}

gint expose_event(GtkWidget *widget,GdkEventExpose *event,GameControl** gamec) {

	gint x,y;

	if(event->send_event == FALSE) { /* == if the expose isn't synthesized by gtk_widget_draw() */

		for(y = (AREA_OY+(event->area.y/TILE_SIZE));y<=(AREA_OY+((event->area.y+event->area.height)/TILE_SIZE));y++)
		for(x = (AREA_OX+(event->area.x/TILE_SIZE));x<=(AREA_OX+((event->area.x+event->area.width)/TILE_SIZE));x++) {

			add_update(gamec,x,y);

		}
		

	}

	return TRUE;

}


gint configure_event(GtkWidget *widget,GdkEventConfigure *event,GameControl** gamec) {

	gint x,y;

	if(MAP_AREA != NULL) {

		gdk_pixmap_unref(MAP_AREA);

	} 

	MAP_AREA = gdk_pixmap_new(widget->window,widget->allocation.width,widget->allocation.height,-1);
	
	for(y = AREA_OY;y<=(AREA_OY+(widget->allocation.height/TILE_SIZE));y++)
	for(x = AREA_OX;x<=(AREA_OX+(widget->allocation.width/TILE_SIZE));x++) {

		add_update(gamec,x,y);

	}

	return TRUE;

}
            
void destroy(GtkWidget *widget,gpointer data) {

	gtk_main_quit();

}

void about(GameControl** gamec) {

	GtkWidget* about;
	GtkWidget* button;
	GtkWidget* label;
	GtkWidget* pixmap1;
	GtkWidget* pixmap2;
	GtkWidget* hbox;

	about = gtk_dialog_new();
	gtk_widget_show(about);

	hbox = gtk_hbox_new(FALSE,0);
	gtk_widget_ref (hbox);
	gtk_object_set_data_full(GTK_OBJECT(about),"hbox",hbox,(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(hbox);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(about)->vbox),hbox,TRUE, TRUE, 0);

	pixmap1 = gtk_pixmap_new(PIXMAP(OGRE_DOWN_WALK_RIGHT_1),MASK(OGRE_DOWN_WALK_RIGHT_1));
	gtk_widget_ref(pixmap1);
	gtk_object_set_data_full(GTK_OBJECT(about),"pixmap1",pixmap1,(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(pixmap1);
	gtk_box_pack_start(GTK_BOX(hbox),pixmap1,TRUE,TRUE,0);
	gtk_misc_set_alignment(GTK_MISC (pixmap1),1,0.5);

	pixmap2 = gtk_pixmap_new(PIXMAP(KNIGHT_DOWN_WALK_LEFT_1),MASK(KNIGHT_DOWN_WALK_LEFT_1));
	gtk_widget_ref(pixmap2);
	gtk_object_set_data_full(GTK_OBJECT(about),"pixmap2",pixmap2,(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show(pixmap2);
	gtk_box_pack_start(GTK_BOX(hbox),pixmap2,TRUE,TRUE,0);
	gtk_misc_set_alignment(GTK_MISC(pixmap2),0,0.5);

	label = gtk_label_new(ABOUT_STRING);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(about)->vbox),label,FALSE,FALSE,0);
	gtk_misc_set_padding(GTK_MISC(label),30,5);
	gtk_widget_show(label);

	button = gtk_button_new_with_label("Ok");
	gtk_signal_connect(GTK_OBJECT(button),"clicked",GTK_SIGNAL_FUNC(about_close),GTK_WIDGET(about));
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(about)->action_area),button,TRUE,TRUE,0);
	gtk_widget_show(button);
}

void about_close(GtkWidget *widget,gpointer data) {
	
	gtk_widget_destroy(GTK_WIDGET(data));

}
